/*****************************************************************************
*
* Copyright (c) 2000 - 2013, Lawrence Livermore National Security, LLC
* Produced at the Lawrence Livermore National Laboratory
* LLNL-CODE-442911
* All rights reserved.
*
* This file is  part of VisIt. For  details, see https://visit.llnl.gov/.  The
* full copyright notice is contained in the file COPYRIGHT located at the root
* of the VisIt distribution or at http://www.llnl.gov/visit/copyright.html.
*
* Redistribution  and  use  in  source  and  binary  forms,  with  or  without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of  source code must  retain the above  copyright notice,
*    this list of conditions and the disclaimer below.
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this  list of  conditions  and  the  disclaimer (as noted below)  in  the
*    documentation and/or other materials provided with the distribution.
*  - Neither the name of  the LLNS/LLNL nor the names of  its contributors may
*    be used to endorse or promote products derived from this software without
*    specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT  HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR  IMPLIED WARRANTIES, INCLUDING,  BUT NOT  LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND  FITNESS FOR A PARTICULAR  PURPOSE
* ARE  DISCLAIMED. IN  NO EVENT  SHALL LAWRENCE  LIVERMORE NATIONAL  SECURITY,
* LLC, THE  U.S.  DEPARTMENT OF  ENERGY  OR  CONTRIBUTORS BE  LIABLE  FOR  ANY
* DIRECT,  INDIRECT,   INCIDENTAL,   SPECIAL,   EXEMPLARY,  OR   CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT  LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR
* SERVICES; LOSS OF  USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER
* CAUSED  AND  ON  ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT
* LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING IN ANY  WAY
* OUT OF THE  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
*
*****************************************************************************/

#ifndef AXISATTRIBUTES_H
#define AXISATTRIBUTES_H
#include <state_exports.h>
#include <AttributeSubject.h>

#include <AxisTitles.h>
#include <AxisLabels.h>
#include <AxisTickMarks.h>

// ****************************************************************************
// Class: AxisAttributes
//
// Purpose:
//    Contains the properties for one axis.
//
// Notes:      Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

class STATE_API AxisAttributes : public AttributeSubject
{
public:
    // These constructors are for objects of this class
    AxisAttributes();
    AxisAttributes(const AxisAttributes &obj);
protected:
    // These constructors are for objects derived from this class
    AxisAttributes(private_tmfs_t tmfs);
    AxisAttributes(const AxisAttributes &obj, private_tmfs_t tmfs);
public:
    virtual ~AxisAttributes();

    virtual AxisAttributes& operator = (const AxisAttributes &obj);
    virtual bool operator == (const AxisAttributes &obj) const;
    virtual bool operator != (const AxisAttributes &obj) const;
private:
    void Init();
    void Copy(const AxisAttributes &obj);
public:

    virtual const std::string TypeName() const;
    virtual bool CopyAttributes(const AttributeGroup *);
    virtual AttributeSubject *CreateCompatible(const std::string &) const;
    virtual AttributeSubject *NewInstance(bool) const;

    // Property selection methods
    virtual void SelectAll();
    void SelectTitle();
    void SelectLabel();
    void SelectTickMarks();

    // Property setting methods
    void SetTitle(const AxisTitles &title_);
    void SetLabel(const AxisLabels &label_);
    void SetTickMarks(const AxisTickMarks &tickMarks_);
    void SetGrid(bool grid_);

    // Property getting methods
    const AxisTitles    &GetTitle() const;
          AxisTitles    &GetTitle();
    const AxisLabels    &GetLabel() const;
          AxisLabels    &GetLabel();
    const AxisTickMarks &GetTickMarks() const;
          AxisTickMarks &GetTickMarks();
    bool                GetGrid() const;

    // Persistence methods
    virtual bool CreateNode(DataNode *node, bool completeSave, bool forceAdd);
    virtual void SetFromNode(DataNode *node);


    // Keyframing methods
    virtual std::string               GetFieldName(int index) const;
    virtual AttributeGroup::FieldType GetFieldType(int index) const;
    virtual std::string               GetFieldTypeName(int index) const;
    virtual bool                      FieldsEqual(int index, const AttributeGroup *rhs) const;


    // IDs that can be used to identify fields in case statements
    enum {
        ID_title = 0,
        ID_label,
        ID_tickMarks,
        ID_grid,
        ID__LAST
    };

private:
    AxisTitles    title;
    AxisLabels    label;
    AxisTickMarks tickMarks;
    bool          grid;

    // Static class format string for type map.
    static const char *TypeMapFormatString;
    static const private_tmfs_t TmfsStruct;
};
#define AXISATTRIBUTES_TMFS "aaab"

#endif
